package com.ekexiu.portal.dao;

import com.ekexiu.portal.po.Ppaper;
import com.ekexiu.portal.pojo.AssedPaper;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.Exclude;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.Or;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.operator.PageQuery;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Alias;
import org.jfw.apt.orm.annotation.dao.param.In;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.annotation.dao.param.SqlColumn;
import org.jfw.apt.orm.core.defaultImpl.FixLenStringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

@DAO
public abstract class PpaperDao {

	@SelectOne
	@Nullable
	public abstract	Ppaper query(Connection con, String id) throws SQLException;

	@SelectList
	public abstract List<Ppaper> query(Connection con, @In String[] id) throws SQLException;

	@UpdateWith
	@From(Ppaper.class)
	public abstract int update(Connection con, String id, @Set @Nullable String keywords) throws SQLException;

	
	@Exclude("summary")
	@PageQuery
	@OrderBy(" ORDER BY ASS_TIME DESC,ID ASC")
	public abstract PageQueryResult<AssedPaper> pageQueryWithProfessor(Connection con,
			@SqlColumn(handlerClass = FixLenStringHandler.class, value = { "A.PROFESSOR_ID=?" })  String professorId,
			@SqlColumn(handlerClass = FixLenStringHandler.class, value = { "P.NAME LIKE ?" }) String name, int pageSize, int pageNo) throws SQLException;

	@Exclude("summary")
	@PageQuery
	@OrderBy(" ORDER BY ASS_TIME DESC,ID ASC")
	public abstract PageQueryResult<AssedPaper> pageQueryWithAuthor(Connection con,
			@SqlColumn(handlerClass = FixLenStringHandler.class, value = { "A.NAME=?" }) String author,
			@SqlColumn(handlerClass = FixLenStringHandler.class, value = { "((A.PROFESSOR_ID=?)  OR  (A.PROFESSOR_ID='################################'))" }) String id,
			@SqlColumn(handlerClass = FixLenStringHandler.class, value = { "P.NAME LIKE ?" }) String name, int pageSize, int pageNo) throws SQLException;
	
	@From(Ppaper.class)
	@UpdateWith
	@SetSentence("PAGE_VIEWS = PAGE_VIEWS + 1")
	public abstract int incPageViews(Connection con,String id)throws SQLException;
	
	@Exclude("summary")
	@PageSelect
	@OrderBy(" ORDER BY SORT_NUM DESC,CREATE_TIME DESC,ID DESC")
	public abstract PageQueryResult<Ppaper> query(Connection con,int pageSize,int pageNo) throws SQLException;
	
	
	@Exclude("summary")
	@PageSelect
	@OrderBy(" ORDER BY SORT_NUM DESC,CREATE_TIME DESC,ID DESC")
	@Or
	public abstract PageQueryResult<Ppaper> query(Connection con,@Alias({"name","authors","keywords","cn4periodical","en4periodical"}) @Like String cnt,int pageSize,int pageNo) throws SQLException;

	public List<Ppaper> indexSearch(Connection con, String key, long sortNum, String createTime, String id, int rows) throws SQLException {
		List<Ppaper> result = new ArrayList<>();
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT AUTHORS,SORT_NUM,ID,NAME,PAGE_VIEWS,CN4PERIODICAL,EN4PERIODICAL,PERIODICALTYPE,PUB_DAY,KEYWORDS,REF_PARAM,SHARE_ID,CREATE_TIME FROM PPAPER");
		sql.append(" WHERE (SORT_NUM < ? OR (SORT_NUM = ? AND CREATE_TIME < ?) OR (SORT_NUM=? AND CREATE_TIME = ? AND ID < ?)) ");
		if (key != null) {
			sql.append(" AND (EN4PERIODICAL LIKE ? OR CN4PERIODICAL LIKE ? OR KEYWORDS LIKE ? OR AUTHORS LIKE ? OR NAME LIKE ?) ");
		}
		sql.append(" ORDER BY SORT_NUM DESC,CREATE_TIME DESC,ID DESC LIMIT ? ");
		PreparedStatement ps = con.prepareStatement(sql.toString());
		try {
			int index = 1;
			ps.setLong(index++, sortNum);
			ps.setLong(index++, sortNum);
			ps.setString(index++, createTime);
			ps.setLong(index++, sortNum);
			ps.setString(index++, createTime);
			ps.setString(index++, id);
			if (key != null) {
				ps.setString(index++, key);
				ps.setString(index++, key);
				ps.setString(index++, key);
				ps.setString(index++, key);
				ps.setString(index++, key);
			}
			ps.setInt(index++, rows);
			ResultSet rs = ps.executeQuery();
			try {
				while (rs.next()) {
					Ppaper ppaper = new Ppaper();
					ppaper.setAuthors(rs.getString(1));
					ppaper.setSortNum(rs.getLong(2));
					ppaper.setId(rs.getString(3));
					ppaper.setName(rs.getString(4));
					ppaper.setPageViews(rs.getLong(5));
					String cn4periodical = rs.getString(6);
					if (rs.wasNull()) {
						cn4periodical = null;
					}
					ppaper.setCn4periodical(cn4periodical);
					String en4periodical = rs.getString(7);
					if (rs.wasNull()) {
						en4periodical = null;
					}
					ppaper.setEn4periodical(en4periodical);
					String periodicaltype = rs.getString(8);
					if (rs.wasNull()) {
						periodicaltype = null;
					}
					ppaper.setPeriodicaltype(periodicaltype);
					ppaper.setPubDay(rs.getString(9));
					String keywords = rs.getString(10);
					if (rs.wasNull()) {
						keywords = null;
					}
					ppaper.setKeywords(keywords);
					String refParam = rs.getString(11);
					if (rs.wasNull()) {
						refParam = null;
					}
					ppaper.setRef_param(refParam);
					ppaper.setShareId(rs.getLong(12));
					ppaper.setCreateTime(rs.getString(13));
					result.add(ppaper);
				}
				return result;
			} finally {
				try {
					rs.close();
				} catch (Exception _m_16) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception _m_17) {
			}
		}
	}


	@SelectOne
	@Nullable
	public abstract Ppaper query(Connection con,long shareId)throws SQLException;
	
	public String[] queryPaperIdWithSameKeyWord(Connection con,String id,int rows)throws SQLException{
		List<String> ret = new ArrayList<String>();
		String sql = "select id,count(1) sort from pap_key_word where kw in (select kw from pap_key_word where id=?) and id <>? group by id order by sort desc limit ?";
		PreparedStatement ps = con.prepareStatement(sql);
		try{
			ps.setString(1, id);
			ps.setString(2, id);
			ps.setInt(3, rows);
			ResultSet rs = ps.executeQuery();
			try{
				while(rs.next()){
					ret.add(rs.getString(1));
				}
			}finally{
				try{rs.close();}catch(Exception e1){}
			}
		}finally{
			try{ps.close();}catch(Exception e2){}
		}
		return ret.isEmpty()?null:ret.toArray(new String[ret.size()]);		
	}
	
	public String[] queryPatentIdWithKeyWord(Connection con, String[] kws, int rows) throws SQLException {
		List<String> ret = new ArrayList<String>();
		StringBuilder sql = new StringBuilder();
		sql.append("select id,count(1) sort from pap_key_word where kw in (");
		for (int i = 0; i < kws.length; ++i) {
			sql.append("?,");
		}
		sql.delete(sql.length() - 1, sql.length());
		sql.append(") group by id order by sort desc limit ?");
		PreparedStatement ps = con.prepareStatement(sql.toString());
		try {
			int idx = 1;
			for (String kw : kws) {
				ps.setString(idx++, kw);
			}
			ps.setInt(idx++, rows);
			ResultSet rs = ps.executeQuery();
			try {
				while (rs.next()) {
					ret.add(rs.getString(1));
				}
			} finally {
				try {
					rs.close();
				} catch (Exception e1) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception e2) {
			}
		}
		return ret.isEmpty() ? null : ret.toArray(new String[ret.size()]);
	}
}
